#ifndef PIN_H
#define PIN_H

#include <stdint.h>
#include <stdbool.h>

#include "bit.h"

#ifdef __cplusplus
extern "C"
{
#endif

typedef enum
{
    PIN_NULL = 0,      // NULL
    PIN_0 = 8 * 1 + 0, // PA0/ADC0
    PIN_1 = 8 * 1 + 1, // PA1/ADC1
    PIN_2 = 8 * 1 + 2, // PA2/ADC2
    PIN_3 = 8 * 1 + 3, // PA3/ADC3
    PIN_4 = 8 * 2 + 3, // PB3/OC0A
    PIN_5 = 8 * 4 + 5, // PD5/OC1A
    PIN_6 = 8 * 4 + 7, // PD7/OC2A
    PIN_7 = 8 * 4 + 6, // PD6/OC2B
    PIN_8 = 8 * 2 + 4, // PB4/SS
    PIN_9 = 8 * 2 + 5, // PB5/MOSI
    PIN_A = 8 * 2 + 6, // PB6/MISO
    PIN_B = 8 * 2 + 7, // PB7/SCK
    PIN_C = 8 * 3 + 0, // PC0/SCL
    PIN_D = 8 * 3 + 1, // PC1/SDA
    PIN_E = 8 * 4 + 0, // PD0/RXD
    PIN_F = 8 * 4 + 1, // PD1/TXD
} pin_t;

typedef enum
{
    INPUT = 0,
    OUTPUT = 1,
} pin_mode_t;

void pin_mode(pin_t _pin, pin_mode_t _mode);

#define LOW    0
#define HIGH   1
#define PULLUP 1

void pin_write(pin_t _pin, bool _level);

#define PIN_BIT(p) ((p) & BIT_MASK(3, 0))
void pin_write_multi(pin_t _pin, uint8_t _mask, uint8_t _value);

void pin_flip(pin_t _pin);

bool pin_read(pin_t _pin);

#ifdef __cplusplus
}
#endif

#endif